@@ -21,7 +21,7 @@ while (<CONF_FILE>)
 	    print HEADER_FILE "#undef DEBUG\n"  if ($2 eq "NO");
 	    print HEADER_FILE "#define DEBUG\n" if ($2 eq "YES");
 	}
-	else # For settings other than DEBUG
+	elsif ($1 ne "UNCOVER_DIRS") # For settings other than DEBUG and UNCOVER_DIRS
 	{
 	    print HEADER_FILE "#define $1 "; # Begin macro definition
 	    
