@@ -1,4 +1,4 @@
-/* Copyright 2001, 2002, 2003 Manuel Arriaga
+/* Copyright 2001, 2002, 2003, 2004 Manuel Arriaga
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -751,14 +751,24 @@ char* build_absolute_path(const char *path)
      dirname = get_current_dir_name();
    else /* if path contains a directory name: */
      {
-	dirname = malloc(slash - path + 1); /* (1) malloc() instead of alloca() so that dirname is always
-					     free()d, independently of that memory having been allocated inside
-					     get_current_dir_name() or by us; (2) no byte for trailing slash */
-
-	if (dirname)
+	if (slash == path) /* path has the form "/fsdds.txt", and dirname is a single "/" */
 	  {
-	     strncpy(dirname, path, slash - path);
-	     dirname[slash - path] = '\0';
+	     dirname = malloc(2);
+
+	     if (dirname)
+	       strcpy(dirname, "/");
+	  }
+	else /* path has the form "/fds/fds.txt", and dirname is "/fds" */
+	  {
+	     dirname = malloc(slash - path + 1); /* (1) malloc() instead of alloca() so that dirname is always
+						  free()d, independently of that memory having been allocated inside
+						  get_current_dir_name() or by us; (2) no byte for trailing slash */
+
+	     if (dirname)
+	       {
+		  strncpy(dirname, path, slash - path);
+		  dirname[slash - path] = '\0';
+	       }
 	  }
      }
 
@@ -783,8 +793,10 @@ char* build_absolute_path(const char *path)
 	     if (absolute_path)
 	       {
 		  strcpy(absolute_path, abs_dirname);
-		  strcat(absolute_path, "/");
 
+		  if (strlen(abs_dirname) > 1)  /* only append an extra slash if abs_dirname is something other than a single slash... */
+		    strcat(absolute_path, "/");
+		  
 		  if (slash)
 		    strcat(absolute_path, slash + 1);
 		  else /* (!slash) */
@@ -1545,12 +1557,12 @@ int decide_action(const char *absolute_path, config *cfg)
 	(!found_under_dir(absolute_path, cfg->home) && !cfg->global_protection) ||      /* is outside of the user's dir and
 											 the user doesn't want us to protect
 											 these files. */
-	
+
 	ends_in_ignored_extension(absolute_path, cfg)                           ||      /* filename ends in an extension
 											 * we were told to ignore */
-	
+
 	(strlen(cfg->ignore_re) > 0 && matches_re(absolute_path, cfg->ignore_re)) ||   /* file name matches the IGNORE_RE */
-	
+
 	found_under_dir(absolute_path, cfg->removable_media_mount_points)       ||      /* file is on a removable medium */
 
 	is_empty_file(absolute_path, cfg)                                       )       /* zero byte-count in regular file */
@@ -1886,7 +1898,7 @@ char * convert_relative_into_absolute_paths(const char *relative_paths)
 	else //if (*orig_ptr == ';')
 	  {
 	     *new_ptr++ = ';';
-	     strcat(new_ptr, userinfo->pw_dir);
+	     strcpy(new_ptr, userinfo->pw_dir);
 	     new_ptr += strlen(userinfo->pw_dir);
 	     *new_ptr = '/';
 	  }
@@ -1937,7 +1949,7 @@ static int is_empty_file(const char *path, config *cfg)
    return retval;
 }
 
-/* The following two user-contributed functions implement support for the IGNORE_RE 
+/* The following two user-contributed functions implement support for the IGNORE_RE
  feature. */
 
 #ifdef DEBUG
@@ -1957,7 +1969,7 @@ static int matches_re (const char *absolute_path, const char *regexp)
    regmatch_t matches[1];
    regex_t    compiled;
    int        ret;
-   
+
    ret = regcomp (&compiled, regexp, REG_EXTENDED);
 
    if (ret)
