@@ -1,4 +1,4 @@
-/* Copyright 2001, 2002, 2003 Manuel Arriaga
+/* Copyright 2001, 2002, 2003, 2004 Manuel Arriaga
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -239,26 +239,36 @@ void init(config *cfg)
 
    /* Get a pointer to GNU libc's original functions: */
 
+   dlerror();
+				
    cfg->real_unlink = dlsym(RTLD_NEXT, "unlink");
 
    if (!dlerror())
      cfg->real_unlink_unavailable = NO;
 
+   dlerror();
+   
    cfg->real_rename = dlsym(RTLD_NEXT, "rename");
 
    if (!dlerror())
      cfg->real_rename_unavailable = NO;
-
-   cfg->real_fopen = dlsym(RTLD_NEXT, "fopen");
-
+   
+   dlerror();
+   
+   cfg->real_fopen = dlvsym(RTLD_NEXT, "fopen", "GLIBC_2.1"); /* funny interaction fixed! when dlsym() was used instead of dlvsym(), GNU libc would give us a pointer to an older version of fopen() and subsequently crash if the calling code tried to use, e.g., getwc(). */
+   
    if (!dlerror())
      cfg->real_fopen_unavailable = NO;
 
+   dlerror();
+   
    cfg->real_freopen = dlsym(RTLD_NEXT, "freopen");
 
    if (!dlerror())
      cfg->real_freopen_unavailable = NO;
 
+   dlerror();
+   
    cfg->real_open = dlsym(RTLD_NEXT, "open");
 
    if (!dlerror())
