@@ -1,4 +1,4 @@
-/* Copyright 2001, 2002, 2003 Manuel Arriaga
+/* Copyright 2001, 2002, 2003, 2004 Manuel Arriaga
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,10 +17,8 @@
  *
  */
 
-/* These file defines the wrappers for the GNU libc functions fopen(), freopen() and
- * open(). Since they are all pretty much the same, and following the
- * approach first used in the libindex component of paise
- * (http://m-arriaga.net/software/paise), they now all invoke a function
+/* This file defines the wrappers for the GNU libc functions fopen(), freopen() and
+ * open(). Since they are all pretty much the same, they all invoke a function
  * called do_fopen_or_freopen_or_open() which emulates each of these
  * functions according to its first argument, a flag which indicates which
  * function it should behave as.
@@ -388,6 +386,8 @@ int open64(const char *path, int flags, ...)
  * we return an error code: (the code isn't delimited by brackets because the pointers need to be available
  * to the rest of the function) */
 
+/* the call to dlvsym() in the particular case of fopen() fixes a funny interaction with GNU libc. When dlsym() was used instead of dlvsym(), GNU libc would give us a pointer to an older version of fopen() and subsequently crash if the calling code tried to run, e.g., getwc() on the returned FILE* pointer. */
+
 #define GET_REAL_FUNCTION()                                                            \
    int (*real_fopen) (const char *path, const char *mode) = NULL;                      \
    int (*real_fopen64) (const char *path, const char *mode) = NULL;                    \
@@ -396,8 +396,10 @@ int open64(const char *path, int flags, ...)
    int (*real_open) (const char *path, int flags, ...) = NULL;                         \
    int (*real_open64) (const char *path, int flags, ...) = NULL;                       \
    \
+   dlerror();                                                                       \
+   \
    if (function == FOPEN)                                                           \
-     real_fopen = dlsym(RTLD_NEXT, "fopen");                                        \
+     real_fopen = dlvsym(RTLD_NEXT, "fopen", "GLIBC_2.1");                          \
    else if (function == FOPEN64)                                                    \
      real_fopen64 = dlsym(RTLD_NEXT, "fopen64");                                    \
    else if (function == FREOPEN)                                                    \
@@ -480,15 +482,15 @@ static FdOrFp do_fopen_or_freopen_or_open(int function, const char *path, ...)
    fprintf(stderr, "Entering do_fopen_or_freopen_or_open(). Arg: %s.\n", path);
 #endif
 
-   /* Do our arguments specify write-mode with orders to truncate existing files? If they don't, we have nothing
+   /* Do our arguments specify write-mode with orders to truncate existing files? If they don't (or if we were passed a NULL argument), we have nothing
     * to do:
     */
 
-   if (!WRITE_TRUNCATE_MODE_ENABLED())
+   if (!WRITE_TRUNCATE_MODE_ENABLED() || path == NULL)
      {
 #ifdef DEBUG
 	fprintf(stderr, "Passing request to the real %s because the arguments we were passed don't specify "
-		"write-mode with orders to truncate existing files.\n", function_name);
+		"write-mode with orders to truncate existing files (or we have a NULL file name).\n", function_name);
 #endif
 	RETURN_REAL_FUNCTION(); /* we don't need to call fini() here. */
      }
