@@ -1,4 +1,4 @@
-/* Copyright 2001, 2002, 2003 Manuel Arriaga
+/* Copyright 2001, 2002, 2003, 2004 Manuel Arriaga
  * 
  * 
  * This program is free software; you can redistribute it and/or modify 
@@ -87,12 +87,13 @@ int rename(const char *oldpath, const char *newpath)
    
    
    /* If libtrash_off is set to true or intercept_rename is set to false, the user has asked us to become temporarily inactive an let the
-    * real rename() perform its task.*/
+    * real rename() perform its task. Alternatively, we might have been passed a NULL pointer and let the real rename handle that:*/
    
-   if (cfg.libtrash_off || !cfg.intercept_rename)
+   if (cfg.libtrash_off || !cfg.intercept_rename || 
+       oldpath == NULL || newpath == NULL)
      {
 #ifdef DEBUG
-	fprintf(stderr, "Passing request to rename(%s, %s) to the real rename() because libtrash_off = true or intercept_rename = false.\n", 
+	fprintf(stderr, "Passing request to rename(%s, %s) to the real rename() because libtrash_off = true or intercept_rename = false (OR: one of the args is NULL).\n", 
 		oldpath, newpath);
 #endif
 	fini(&cfg);
