@@ -1,4 +1,4 @@
-/* Copyright 2001, 2002, 2003 Manuel Arriaga
+/* Copyright 2001, 2002, 2003, 2004 Manuel Arriaga
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -83,15 +83,19 @@ int unlink(const char *pathname)
      }
 
    /* If libtrash_off is set to true or intercept_unlink set to false, the user has asked us to become temporarily inactive an let the real
-    * unlink() perform its task.*/
+    * unlink() perform its task.
+    Alternatively, if we were passed a NULL pointer we also call the real unlink: */
 
-   if (cfg.libtrash_off || !cfg.intercept_unlink)
+   if (cfg.libtrash_off || !cfg.intercept_unlink || pathname == NULL)
      {
 #ifdef DEBUG
-	fprintf(stderr, "Passing request to unlink %s to the real unlink because libtrash_off = true or intercept_unlink = false.\n", pathname);
+       if (!pathname)
+	 fprintf(stderr, "unlink was passed a NULL pointer, calling real unlink.\n");
+       else
+	 fprintf(stderr, "Passing request to unlink %s to the real unlink because libtrash_off = true or intercept_unlink = false.\n", pathname);
 #endif
-	fini(&cfg);
-	return (*cfg.real_unlink) (pathname); /* real unlink() sets errno */
+       fini(&cfg);
+       return (*cfg.real_unlink) (pathname); /* real unlink() sets errno */
      }
 
    /* If general_failure is set, something went wrong while initializing and we should just invoke the real function: */
